/*
 * Copyright (C) 2021, Xilinx Inc - All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

#pragma once

#include <memory>
#include "xma.h"
#include "SideData.h"

namespace xma {
//////////////////////////////////////////////////////////////////////////////
class FrameProperties {
protected:
  FrameProperties() = default;

public:
  virtual ~FrameProperties() = default;

  virtual enum XmaFormatType GetFormat() const                   = 0;
  virtual enum XmaFormatType GetSwFormat() const                 = 0;
  virtual int32_t            GetWidth() const                    = 0;
  virtual int32_t            GetHeight() const                   = 0;
  virtual uint32_t           GetFlags() const                    = 0;
  virtual int32_t            GetNumPlanes() const                = 0;
  virtual int32_t            GetPlaneHeight(int32_t plane) const = 0;
  virtual int32_t            GetPlaneStride(int32_t plane) const = 0;
  virtual int32_t            GetPlaneSize(int32_t plane) const   = 0;
};

using FramePropertiesPtr = std::shared_ptr<FrameProperties>;

class Frame : public std::enable_shared_from_this<Frame> {
protected:
  Frame() = default;

public:
  virtual ~Frame() = default;

  virtual XmaBufferType      GetBufferType() const                = 0;
  virtual void*              GetBuffer(int32_t plane) const       = 0;
  virtual uint8_t*           GetHostBuffer(int32_t plane) const   = 0;
  virtual void*              GetDeviceBuffer(int32_t plane) const = 0;
  virtual FramePropertiesPtr GetProps() const                     = 0;
  virtual XmaFraction        GetFrameRate() const                 = 0;
  virtual int64_t            GetPts() const                       = 0;
  virtual int                IsIdr() const                        = 0;

  virtual int32_t     IncRef()                                           = 0;
  virtual int32_t     DecRef()                                           = 0;
  virtual int32_t     GetRefCount()                                      = 0;
  virtual int32_t     AddSideData(SideDataPtr side_data)                 = 0;
  virtual SideDataPtr GetSideData(enum XmaFrameSideDataType type)        = 0;
  virtual int32_t     RemoveSideData(SideDataPtr side_data)              = 0;
  virtual int32_t     RemoveSideDataType(enum XmaFrameSideDataType type) = 0;
  virtual void        ClearAllSideData()                                 = 0;
};

using FramePtr = std::shared_ptr<Frame>;

}; // Namespace xma
