/*
 * Copyright (C) 2021, Xilinx Inc - All rights reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may
 * not use this file except in compliance with the License. A copy of the
 * License is located at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

#pragma once

#include <mutex>
#include <functional>
#include <condition_variable>
#include "Exception.h"
#include "XmaLog.h"
#ifdef __cplusplus
extern "C" {
#endif
#include "vpi_buffer.h"
#ifdef __cplusplus
}
#endif

namespace xma {

#define BUFFER_POOL_AUTO_EXTEND_DEFAULT_SIZE 16
#define BUFFER_POOL_AUTO_EXTEND_SIZE 16

//////////////////////////////////////////////////////////////////////////////
using buffer_t = std::unique_ptr<VpiBuffer, std::function<void(VpiBuffer* buf)>>;

class BufferPool {
public:
  BufferPool(VpiMemHandle mem, LogPtr log, uint32_t num, uint32_t size, uint32_t flag = RC_SIDE_EN | EP_SIDE_EN, bool auto_extend = false);
  virtual ~BufferPool();

  int32_t SetLog(LogPtr log);

  buffer_t       GetBuffer();
  VpiBuffer*     GetBufferAt(uint32_t index) const;
  VpiBufferPool* GetVpiPool() const;
  void           FreeBuffer(VpiBuffer* buffer);
  int32_t        Extend(uint32_t num);
  uint32_t       GetCurrentSize() const;

protected:
  void InitData();

protected:
  using lock_t = std::unique_lock<std::mutex>;
  using pool_t = std::unique_ptr<VpiBufferPool, std::function<void(VpiBufferPool* pool)>>;
  std::mutex              m_mutex;
  pool_t                  m_pool;
  uint32_t                m_size               = 0;
  int32_t                 m_wait_for_empty_buf = 0;
  std::condition_variable m_cond;
  LogPtr                  m_log;
  bool                    m_auto_extend;
};

using BufferPoolPtr = std::shared_ptr<BufferPool>;

}; // Namespace xma
